@@ -1,3 +1,6 @@
+# Copyright 2018-2021 the .NET Foundation
+# Licensed under the BSD license
+
 import os
 from datetime import datetime
 
@@ -19,20 +22,22 @@ except ImportError:
 else:
     OPENGL_INSTALLED = True
 
-if QT_INSTALLED and OPENGL_INSTALLED:
+RUNNING_ON_CI = os.environ.get('CI') or os.environ.get('AGENT_OS')
 
+if QT_INSTALLED and OPENGL_INSTALLED:
     from qtpy.QtWidgets import QOpenGLWidget
 
     class OpenGLWidget(QOpenGLWidget):
-
         def getOpenGLInfo(self):
-
-            gl_parameters = {'vendor': gl.GL_VENDOR,
-                             'renderer': gl.GL_RENDERER,
-                             'version': gl.GL_VERSION,
-                             'shader': gl.GL_SHADING_LANGUAGE_VERSION}
+            gl_parameters = {
+                'vendor': gl.GL_VENDOR,
+                'renderer': gl.GL_RENDERER,
+                'version': gl.GL_VERSION,
+                'shader': gl.GL_SHADING_LANGUAGE_VERSION
+            }
 
             info = {}
+
             for key, setting in gl_parameters.items():
                 try:
                     info[key] = gl.glGetString(setting).decode('ascii')
@@ -42,19 +47,10 @@ if QT_INSTALLED and OPENGL_INSTALLED:
             return info
 
 
-_cached_opengl_renderer = ''
-
-
-RUNNING_ON_CI = os.environ.get('CI') or os.environ.get('AGENT_OS')
-
-
 def pytest_report_header(config):
-    global _cached_opengl_renderer
-
     lines = []
 
     if QT_INSTALLED:
-
         # Get Python Qt bindings version
         lines.append("PyQt: {0}".format(qtpy.PYQT_VERSION))
         lines.append("PySide: {0}".format(qtpy.PYSIDE_VERSION))
@@ -64,7 +60,6 @@ def pytest_report_header(config):
         lines.append("Web Framework: {0}".format('WebEngine' if WEBENGINE else 'WebKit'))
 
         if OPENGL_INSTALLED:
-
             # Get OpenGL version
             from .app import get_qapp
             get_qapp()
@@ -77,17 +72,9 @@ def pytest_report_header(config):
             lines.append("OpenGL Renderer: {0}".format(opengl_info['renderer']))
             lines.append("OpenGL Version: {0}".format(opengl_info['version']))
             lines.append("Shader Version: {0}".format(opengl_info['shader']))
-
-            # This is (no surprise) a hack to enable the Windows testing
-            # framework to check which renderer WebKit is using, which affets
-            # the output.
-            _cached_opengl_renderer = opengl_info['renderer']
         else:
-
             lines.append("Could not determine OpenGL version (OpenGL package required)")
-
     else:
-
         lines.append("Qt not installed")
 
     return os.linesep + os.linesep.join(lines) + os.linesep
@@ -102,21 +89,28 @@ def pytest_unconfigure(config):
 REFERENCE_TIME = datetime(2017, 2, 1, 0, 0, 0, 0)
 
 if QT_INSTALLED:
-
-    @pytest.fixture(scope='session')
-    def wwt_qt_client():
-        from .qt import WWTQtClient
-        wwt = WWTQtClient(block_until_ready=True, size=(400, 400))
-        wwt.set_current_time(REFERENCE_TIME)
-        wwt.pause_time()
-        yield wwt
-        wwt.close()
+    def _check_app_available():
+        import os.path
+        f = os.path.join(os.path.dirname(__file__), 'web_static', 'research', 'index.html')
+        if not os.path.exists(f):
+            raise Exception('You must run `python setup.py build` before running these tests (to get research app files)')
 
     @pytest.fixture(scope='function')
     def wwt_qt_client_isolated():
-        from .qt import WWTQtClient
-        wwt = WWTQtClient(block_until_ready=True, size=(400, 400))
+        _check_app_available()
+        from . import qt
+
+        # As mentioned in wait_for_test() in tests/__init__.py, on the CI
+        # system, the macOS tests use software rendering and it is slooooooow.
+        # We need to give a very generous deadline for checking the app's
+        # aliveness since the renders can cause huge amounts of wall-clock time
+        # to elapse.
+        qt.APP_LIVENESS_DEADLINE = 45
+
+        wwt = qt.WWTQtClient(block_until_ready=True, size=(400, 400))
         wwt.set_current_time(REFERENCE_TIME)
         wwt.pause_time()
         yield wwt
         wwt.close()
+
+    wwt_qt_client = wwt_qt_client_isolated
