@@ -1,3 +1,13 @@
+# Copyright 2021 the .NET Foundation
+# Licensed under the three-clause BSD License
+
+"""
+Utilities for dealing with WWT imagery collections.
+
+The code in this module is fairly old and in need of updating. You won't need to
+use its functionality directly if you're not a pywwt developer.
+"""
+
 import re
 
 from io import BytesIO
@@ -6,6 +16,12 @@ from xml.etree.ElementTree import ElementTree
 
 import requests
 
+__all__ = [
+    "get_imagery_layers",
+    "Bandpass",
+    "ImageryLayers",
+]
+
 
 def get_imagery_layers(url):
     """
@@ -28,17 +44,17 @@ def get_imagery_layers(url):
     e = ElementTree()
     t = e.parse(b)
 
-    for survey in t.iter('ImageSet'):
-        name = survey.attrib['Name']
-        thumbnail_url = survey.find('ThumbnailUrl')
+    for survey in t.iter("ImageSet"):
+        name = survey.attrib["Name"]
+        thumbnail_url = survey.find("ThumbnailUrl")
         if thumbnail_url is not None:
-            thumbnail_url = getattr(thumbnail_url, 'text', None) or None
-        available_layers[name] = {'thumbnail': thumbnail_url}
+            thumbnail_url = getattr(thumbnail_url, "text", None) or None
+        available_layers[name] = {"thumbnail": thumbnail_url}
 
     return available_layers
 
 
-class ImageryLayers():
+class ImageryLayers:
     """
     A supplemental class that standardizes layer names, sorts them by
     bandpass, and makes it possible to tab complete them.
@@ -46,10 +62,28 @@ class ImageryLayers():
 
     def __init__(self, layer_list):
         self._layers = {}
-        self._spectrum = ['gamma', 'x', 'uv', 'visible',
-                          'ir', 'micro', 'radio', 'other']
-        self.integers = ['zero', 'one', 'two', 'three', 'four',
-                         'five', 'six', 'seven', 'eight', 'nine']
+        self._spectrum = [
+            "gamma",
+            "x",
+            "uv",
+            "visible",
+            "ir",
+            "micro",
+            "radio",
+            "other",
+        ]
+        self.integers = [
+            "zero",
+            "one",
+            "two",
+            "three",
+            "four",
+            "five",
+            "six",
+            "seven",
+            "eight",
+            "nine",
+        ]
         for band in self._spectrum:
             self._layers[band] = {}
 
@@ -60,44 +94,52 @@ class ImageryLayers():
         # (og_list) into a dict.
         for layer in og_list:
 
-            if re.search(r'(?i)gamma', layer) is not None:
-                self._add2dict(self._layers, layer, 'gamma')
+            if re.search(r"(?i)gamma", layer) is not None:
+                self._add2dict(self._layers, layer, "gamma")
                 continue  # automatically advance to next iteration
 
-            if re.search(r'(?i)x(-|\s)?ray', layer) is not None:
-                self._add2dict(self._layers, layer, 'x')
+            if re.search(r"(?i)x(-|\s)?ray", layer) is not None:
+                self._add2dict(self._layers, layer, "x")
                 continue
 
-            if (re.search(r'(?i)ultra(-|\s)?violet', layer) is not None or
-                    re.search(r'(?i)[^\d\w]+uv|uv[^\d\w]+', layer) is not None):
-                self._add2dict(self._layers, layer, 'uv')
+            if (
+                re.search(r"(?i)ultra(-|\s)?violet", layer) is not None
+                or re.search(r"(?i)[^\d\w]+uv|uv[^\d\w]+", layer) is not None
+            ):
+                self._add2dict(self._layers, layer, "uv")
                 continue
 
-            if (re.search(r'(?i)optical', layer) is not None or
-                    re.search(r'(?i)visible', layer) is not None):
-                self._add2dict(self._layers, layer, 'visible')
+            if (
+                re.search(r"(?i)optical", layer) is not None
+                or re.search(r"(?i)visible", layer) is not None
+            ):
+                self._add2dict(self._layers, layer, "visible")
                 continue
 
-            if (re.search(r'(?i)infrared', layer) is not None or
-                    re.search(r'(?i)[^\d\w]+ir|ir[^\d\w]+', layer) is not None):
-                self._add2dict(self._layers, layer, 'ir')
+            if (
+                re.search(r"(?i)infrared", layer) is not None
+                or re.search(r"(?i)[^\d\w]+ir|ir[^\d\w]+", layer) is not None
+            ):
+                self._add2dict(self._layers, layer, "ir")
                 continue
 
-            if (re.search(r'(?i)microwave', layer) is not None or
-                    re.search(r'(?i)[^\d\w]+cmb|cmb[^\d\w]+', layer) is not None):
-                self._add2dict(self._layers, layer, 'micro')
+            if (
+                re.search(r"(?i)microwave", layer) is not None
+                or re.search(r"(?i)[^\d\w]+cmb|cmb[^\d\w]+", layer) is not None
+            ):
+                self._add2dict(self._layers, layer, "micro")
                 continue
 
-            if re.search(r'(?i)radio', layer) is not None:
-                self._add2dict(self._layers, layer, 'radio')
+            if re.search(r"(?i)radio", layer) is not None:
+                self._add2dict(self._layers, layer, "radio")
                 continue
 
-            self._add2dict(self._layers, layer, 'other')
+            self._add2dict(self._layers, layer, "other")
 
     def _add2dict(self, diction, full_layer, bandpass):
         # Handles a layer's (full_layer) actual addition to the master
         # dict of layers (diction) according to its bandpass.
-        suffix = ''
+        suffix = ""
         short = self._shorten(full_layer) + suffix
 
         while short in diction[bandpass]:
@@ -109,24 +151,24 @@ class ImageryLayers():
                 short += str(suffix)
 
         diction[bandpass][short] = {}
-        diction[bandpass][short]['full_name'] = full_layer
+        diction[bandpass][short]["full_name"] = full_layer
 
     def _shorten(self, string):
         # Unlocks tab completion by shortening a full layer's name
         # (string) to a valid Python name based on its first word.
-        cut_left = re.search(r'^[_\W]+', string)
+        cut_left = re.search(r"^[_\W]+", string)
         if cut_left is not None:
             string = string[cut_left.end():]
 
-        cut_right = re.search(r'[_\W]', string)
+        cut_right = re.search(r"[_\W]", string)
         if cut_right is not None:
-            string = string[:cut_right.start()].lower()
+            string = string[: cut_right.start()].lower()
 
-        digit = re.search(r'^\d', string)
+        digit = re.search(r"^\d", string)
         if digit is not None:
             for i, num in enumerate(self.integers, 0):
                 if str(i) == digit.group(0):
-                    string = string[:digit.start()] + num + string[digit.end():]
+                    string = string[: digit.start()] + num + string[digit.end():]
                     break
 
         return string
@@ -138,7 +180,7 @@ class ImageryLayers():
         return Bandpass(self._layers[band])
 
 
-class Bandpass():
+class Bandpass:
     """
     Allows the __getattr__() method from ImageryLayers to reach into the
     inner dict corresponding to layers of a particular bandpass.
@@ -151,4 +193,4 @@ class Bandpass():
         return sorted(self._band.keys())
 
     def __getattr__(self, name):
-        return self._band[name]['full_name']
+        return self._band[name]["full_name"]
