@@ -10,14 +10,14 @@ from traitlets import Unicode, default, link, directional_link
 from ipyevents import Event as DOMListener
 from ipykernel.comm import Comm
 
-from .core import BaseWWTWidget
+from .core import BaseWWTWidget, DataPublishingNotAvailableError
 from .layers import ImageLayer
 from .logger import logger
-from .jupyter_server import serve_file
+from .jupyter_relay import get_relay_hub
 
-__all__ = ['WWTJupyterWidget', 'WWTLabApplication', 'connect_to_app']
+__all__ = ["WWTJupyterWidget", "WWTLabApplication", "connect_to_app"]
 
-_npm_version = '^1.3.1'  # cranko internal-req npm:pywwt
+_npm_version = "^1.3.2"  # cranko internal-req npm:pywwt
 VIEW_MODULE_VERSION = _npm_version
 MODEL_MODULE_VERSION = _npm_version
 
@@ -40,30 +40,31 @@ class WWTJupyterWidget(widgets.DOMWidget, BaseWWTWidget):
 
     """
 
-    _view_name = Unicode('WWTView').tag(sync=True)
-    _model_name = Unicode('WWTModel').tag(sync=True)
-    _view_module = Unicode('pywwt').tag(sync=True)
-    _model_module = Unicode('pywwt').tag(sync=True)
+    _view_name = Unicode("WWTView").tag(sync=True)
+    _model_name = Unicode("WWTModel").tag(sync=True)
+    _view_module = Unicode("pywwt").tag(sync=True)
+    _model_module = Unicode("pywwt").tag(sync=True)
     _view_module_version = Unicode(VIEW_MODULE_VERSION).tag(sync=True)
     _model_module_version = Unicode(MODEL_MODULE_VERSION).tag(sync=True)
 
-    _appUrl = Unicode('').tag(sync=True)
+    _appUrl = Unicode("").tag(sync=True)
 
     def __init__(self, hide_all_chrome=False):
-        # In the future we might want to make it possible to use the WWT-hosted
-        # app instead of the bundled version.
+        # Set up to serve the bundled app. In the future we might want to make
+        # it possible to use the WWT-hosted app instead of the bundled version.
         #
         # The JS frontend will automagically prepend the Jupyter base URL if
         # needed and turn this into an absolute URL.
 
         _maybe_perpetrate_mega_kernel_hack()
 
-        self._appUrl = '/wwt/research/'
+        hub = get_relay_hub()
+        self._appUrl = hub.get_static_files_url() + "research/"
 
         widgets.DOMWidget.__init__(self)
         dom_listener.source = self
         dom_listener.prevent_default_action = True
-        dom_listener.watched_events = ['wheel']
+        dom_listener.watched_events = ["wheel"]
 
         self._controls = None
 
@@ -83,8 +84,8 @@ class WWTJupyterWidget(widgets.DOMWidget, BaseWWTWidget):
 
         # Special message from the ipywidgets bridge to indicate
         # when the first widget view is ready to accept messages.
-        if content.get('type') == 'wwt_jupyter_widget_status':
-            self._on_app_status_change(alive=content['alive'])
+        if content.get("type") == "wwt_jupyter_widget_status":
+            self._on_app_status_change(alive=content["alive"])
 
         self._on_app_message_received(content)
 
@@ -94,12 +95,12 @@ class WWTJupyterWidget(widgets.DOMWidget, BaseWWTWidget):
         """
         self.send(payload)
 
-    @default('layout')
+    @default("layout")
     def _default_layout(self):
-        return widgets.Layout(height='400px', align_self='stretch')
+        return widgets.Layout(height="400px", align_self="stretch")
 
-    def _serve_file(self, filename, extension=''):
-        return serve_file(filename, extension=extension)
+    def _serve_file(self, filename, extension=""):
+        return get_relay_hub().serve_file(filename, extension=extension)
 
     def _create_image_layer(self, **kwargs):
         """Returns a specialized subclass of ImageLayer that has some extra hooks for
@@ -111,16 +112,21 @@ class WWTJupyterWidget(widgets.DOMWidget, BaseWWTWidget):
     @property
     def layer_controls(self):
         if self._controls is None:
-            opacity_slider = widgets.FloatSlider(value=self.foreground_opacity,
-                                                 min=0, max=1, readout=False)
-            foreground_menu = widgets.Dropdown(options=self.available_layers,
-                                               value=self.foreground)
-            background_menu = widgets.Dropdown(options=self.available_layers,
-                                               value=self.background)
-            link((opacity_slider, 'value'), (self, 'foreground_opacity'))
-            link((foreground_menu, 'value'), (self, 'foreground'))
-            link((background_menu, 'value'), (self, 'background'))
-            self._controls = widgets.HBox([background_menu, opacity_slider, foreground_menu])
+            opacity_slider = widgets.FloatSlider(
+                value=self.foreground_opacity, min=0, max=1, readout=False
+            )
+            foreground_menu = widgets.Dropdown(
+                options=self.available_layers, value=self.foreground
+            )
+            background_menu = widgets.Dropdown(
+                options=self.available_layers, value=self.background
+            )
+            link((opacity_slider, "value"), (self, "foreground_opacity"))
+            link((foreground_menu, "value"), (self, "foreground"))
+            link((background_menu, "value"), (self, "background"))
+            self._controls = widgets.HBox(
+                [background_menu, opacity_slider, foreground_menu]
+            )
         return self._controls
 
 
@@ -137,84 +143,84 @@ class JupyterImageLayer(ImageLayer):
             return self._controls
 
         opacity = widgets.FloatSlider(
-            description='Opacity:',
+            description="Opacity:",
             value=self.opacity,
             min=0,
             max=1,
             readout=False,
             step=0.01,
-            layout={'width': '200px'}
+            layout={"width": "200px"},
         )
-        link((self, 'opacity'), (opacity, 'value'))
+        link((self, "opacity"), (opacity, "value"))
 
         stretch = widgets.Dropdown(
-            description='Stretch:',
+            description="Stretch:",
             options=VALID_STRETCHES,
             value=self.stretch,
-            layout={'width': '200px'}
+            layout={"width": "200px"},
         )
-        link((self, 'stretch'), (stretch, 'value'))
+        link((self, "stretch"), (stretch, "value"))
 
         # NB, this will crash if `self.cmap` is not one of our allowed values
         reverse_ui_colormaps = dict((kv[1], kv[0]) for kv in UI_COLORMAPS.items())
         colormap = widgets.Dropdown(
-            description='Colormap:',
+            description="Colormap:",
             options=UI_COLORMAPS.keys(),
             value=reverse_ui_colormaps[self.cmap.name],
-            layout={'width': '200px'}
+            layout={"width": "200px"},
+        )
+        directional_link((colormap, "label"), (self, "cmap"), lambda x: UI_COLORMAPS[x])
+        directional_link(
+            (self, "cmap"), (colormap, "label"), lambda x: reverse_ui_colormaps[x.name]
         )
-        directional_link((colormap, 'label'), (self, 'cmap'), lambda x: UI_COLORMAPS[x])
-        directional_link((self, 'cmap'), (colormap, 'label'), lambda x: reverse_ui_colormaps[x.name])
 
         vrange = widgets.FloatRangeSlider(
-            description='Fine min/max:',
+            description="Fine min/max:",
             value=[self.vmin, self.vmax],
             min=self._data_min,
             max=self._data_max,
             readout=True,
-            layout={'width': '600px'},
+            layout={"width": "600px"},
             step=(self.vmax - self.vmin) / 100,
-            format='.3g'
+            format=".3g",
         )
 
         # Linkage must be manual since vrange uses a pair of values whereas we
         # have two separate traitlets.
-        vrange.observe(self._vrange_slider_updated, names=['value'])
+        vrange.observe(self._vrange_slider_updated, names=["value"])
 
         def update_vrange(change):
             # Note: when this function is called, these values are indeed updated.
             vrange.value = (self.vmin, self.vmax)
 
-        self.observe(update_vrange, names=['vmin', 'vmax'])
+        self.observe(update_vrange, names=["vmin", "vmax"])
 
         def update_step(change):
             vrange.step = (vrange.max - vrange.min) / 100
 
-        vrange.observe(update_step, names=['min', 'max'])
+        vrange.observe(update_step, names=["min", "max"])
 
         coarse_min = widgets.FloatText(
-            description='Coarse min:',
-            value=self._data_min,
-            layout={'width': '300px'}
+            description="Coarse min:", value=self._data_min, layout={"width": "300px"}
         )
-        link((coarse_min, 'value'), (vrange, 'min'))
+        link((coarse_min, "value"), (vrange, "min"))
 
         coarse_max = widgets.FloatText(
-            description='Coarse max:',
-            value=self._data_max,
-            layout={'width': '300px'}
+            description="Coarse max:", value=self._data_max, layout={"width": "300px"}
         )
-        link((coarse_max, 'value'), (vrange, 'max'))
+        link((coarse_max, "value"), (vrange, "max"))
 
-        self._controls = widgets.VBox([
-            widgets.HBox([colormap, stretch, opacity]),
-            widgets.HBox([coarse_min, coarse_max]),
-            vrange,
-        ])
+        self._controls = widgets.VBox(
+            [
+                widgets.HBox([colormap, stretch, opacity]),
+                widgets.HBox([coarse_min, coarse_max]),
+                vrange,
+            ]
+        )
         return self._controls
 
     def _vrange_slider_updated(self, change):
-        self.vmin, self.vmax = change['new']
+        self.vmin, self.vmax = change["new"]
 
 
 class WWTLabApplication(BaseWWTWidget):
@@ -231,11 +237,12 @@ class WWTLabApplication(BaseWWTWidget):
 
     _comm = None
     _controls = None
+    _relayAvailable = False
 
     def __init__(self):
         _maybe_perpetrate_mega_kernel_hack()
 
-        self._comm = Comm(target_name='@wwtelescope/jupyterlab:research', data={})
+        self._comm = Comm(target_name="@wwtelescope/jupyterlab:research", data={})
         self._comm.on_msg(self._on_comm_message_received)
         self._comm.open()
 
@@ -250,21 +257,42 @@ class WWTLabApplication(BaseWWTWidget):
         they just disappear. I don't know if there's a "right" way to address
         that.
         """
-        payload = msg['content']['data']
+        payload = msg["content"]["data"]
+        ptype = payload.get("type")
 
         # Special message from the hub indicating app liveness status
-        if payload.get('type') == 'wwt_jupyter_viewer_status':
-            self._on_app_status_change(alive=payload['alive'])
+        if ptype == "wwt_jupyter_viewer_status":
+            self._on_app_status_change(alive=payload["alive"])
             # don't return -- maybe someone downstream can use this, and message
             # processing needs to handle all sorts of unexpected messages anyway
+        elif ptype == "wwt_jupyter_startup_info":
+            self._relayAvailable = payload.get("dataRelayConfirmedAvailable", False)
+            return
 
         self._on_app_message_received(payload)
 
     def _actually_send_msg(self, payload):
         self._comm.send(payload)
 
-    def _serve_file(self, filename, extension=''):
-        return serve_file(filename, extension=extension)
+    def _serve_file(self, filename, extension=""):
+        if not self._relayAvailable:
+            raise DataPublishingNotAvailableError(
+                "Unable to complete this operation because it relies on "
+                "data relay services that are not available. Ensure that "
+                "your Jupyter server has the `wwt_kernel_data_relay` package "
+                "installed."
+            )
+        return get_relay_hub().serve_file(filename, extension=extension)
+
+    def _serve_tree(self, path):
+        if not self._relayAvailable:
+            raise DataPublishingNotAvailableError(
+                "Unable to complete this operation because it relies on "
+                "data relay services that are not available. Ensure that "
+                "your Jupyter server has the `wwt_kernel_data_relay` package "
+                "installed."
+            )
+        return get_relay_hub().serve_tree(path)
 
     def _create_image_layer(self, **kwargs):
         """Returns a specialized subclass of ImageLayer that has some extra hooks for
@@ -276,23 +304,28 @@ class WWTLabApplication(BaseWWTWidget):
     @property
     def layer_controls(self):
         if self._controls is None:
-            opacity_slider = widgets.FloatSlider(value=self.foreground_opacity,
-                                                 min=0, max=1, readout=False)
-            foreground_menu = widgets.Dropdown(options=self.available_layers,
-                                               value=self.foreground)
-            background_menu = widgets.Dropdown(options=self.available_layers,
-                                               value=self.background)
-            link((opacity_slider, 'value'), (self, 'foreground_opacity'))
-            link((foreground_menu, 'value'), (self, 'foreground'))
-            link((background_menu, 'value'), (self, 'background'))
-            self._controls = widgets.HBox([background_menu, opacity_slider, foreground_menu])
+            opacity_slider = widgets.FloatSlider(
+                value=self.foreground_opacity, min=0, max=1, readout=False
+            )
+            foreground_menu = widgets.Dropdown(
+                options=self.available_layers, value=self.foreground
+            )
+            background_menu = widgets.Dropdown(
+                options=self.available_layers, value=self.background
+            )
+            link((opacity_slider, "value"), (self, "foreground_opacity"))
+            link((foreground_menu, "value"), (self, "foreground"))
+            link((background_menu, "value"), (self, "background"))
+            self._controls = widgets.HBox(
+                [background_menu, opacity_slider, foreground_menu]
+            )
         return self._controls
 
 
 def connect_to_app():
     """
     Connect to a WWT application running inside a JupyterLab computational
-    environment.
+    environment. This is your preferred gateway to using WWT in JupyterLab.
 
     For the time being, you must have opened the AAS WorldWide Telescope app
     inside JupyterLab. You can do this by clicking the large WWT icon in the
@@ -300,6 +333,15 @@ def connect_to_app():
     You can open the JupyterLab command palette by typing
     Control/Command-Shift-C.
 
+    The traditional way to use WWT in a JupyterLab notebook is with the
+    following commands in their own cell::
+
+        from pywwt.jupyter import connect_to_app
+        wwt = await connect_to_app().becomes_ready()
+
+    Once you have this *wwt* variable, you can control WWT using all of the
+    commands defined on the :class:`~pywwt.jupyter.WWTLabApplication` class.
+
     Returns
     -------
     app : :class:`~pywwt.jupyter.WWTLabApplication`
@@ -368,7 +410,7 @@ def _maybe_perpetrate_mega_kernel_hack():
     try:
         _maybe_perpetrate_mega_kernel_hack_inner()
     except Exception:
-        logger.exception('failed to set up Jupyter kernel async hack')
+        logger.exception("failed to set up Jupyter kernel async hack")
 
 
 def _maybe_perpetrate_mega_kernel_hack_inner():
@@ -379,7 +421,7 @@ def _maybe_perpetrate_mega_kernel_hack_inner():
     kernel = ipykernel.kernelbase.Kernel.instance()
     orig_schedule_dispatch = kernel.schedule_dispatch
 
-    if getattr(kernel, '_pywwt_mega_hack_installed', False):
+    if getattr(kernel, "_pywwt_mega_hack_installed", False):
         return
 
     # If asyncio doesn't think that there's a running event loop, we don't seem
@@ -412,7 +454,7 @@ def _maybe_perpetrate_mega_kernel_hack_inner():
             kernel.log.error("Invalid Expedited Message (pywwt)", exc_info=True)
             return
 
-        msg_type = msg['header']['msg_type']
+        msg_type = msg["header"]["msg_type"]
         handler = kernel.shell_handlers.get(msg_type, None)
 
         if handler is None:
@@ -423,9 +465,13 @@ def _maybe_perpetrate_mega_kernel_hack_inner():
             try:
                 result = handler(kernel.shell_stream, idents, msg)
                 if inspect.isawaitable(result):
-                    kernel.log.error("Expedited message (pywwt) produce an awaitable result")
+                    kernel.log.error(
+                        "Expedited message (pywwt) produce an awaitable result"
+                    )
             except Exception:
-                kernel.log.error("Exception in expedited message handler (pywwt):", exc_info=True)
+                kernel.log.error(
+                    "Exception in expedited message handler (pywwt):", exc_info=True
+                )
 
     def pywwt_schedule_shell_dispatch_with_expedite(*args):
         """
@@ -443,10 +489,14 @@ def _maybe_perpetrate_mega_kernel_hack_inner():
             # We can't deserialize() here: each message can only be deserialized once
             # due to the replay prevention framework.
             peek_content = kernel.session.unpack(msg[4].bytes)
-            expedite_flag = peek_content.get('data', {}).get('content', {}).get('_pywwtExpedite')
+            expedite_flag = (
+                peek_content.get("data", {}).get("content", {}).get("_pywwtExpedite")
+            )
 
             if expedite_flag:
-                kernel.io_loop.add_callback(dispatch_one_expedited_shell_message, idents, msg)
+                kernel.io_loop.add_callback(
+                    dispatch_one_expedited_shell_message, idents, msg
+                )
                 expedited_it = True
         finally:
             if not expedited_it:
@@ -454,4 +504,4 @@ def _maybe_perpetrate_mega_kernel_hack_inner():
 
     kernel.shell_stream.on_recv(pywwt_schedule_shell_dispatch_with_expedite, copy=False)
     kernel._pywwt_mega_hack_installed = True
-    logger.debug('installed Jupyter kernel message expedite hack')
+    logger.debug("installed Jupyter kernel message expedite hack")
